package io.xjar;

import io.xjar.filter.XMixEntryFilter;
import java.util.regex.Pattern;
import java.security.NoSuchAlgorithmException;
import io.xjar.filter.XAllEntryFilter;
import org.apache.commons.compress.archivers.jar.JarArchiveEntry;
import io.xjar.filter.XAnyEntryFilter;
import io.xjar.key.XKey;
import java.io.File;

public class XDecryption
{
    private File xJar;
    private XKey key;
    private XAnyEntryFilter<JarArchiveEntry> includes;
    private XAllEntryFilter<JarArchiveEntry> excludes;
    
    public XDecryption() {
        this.includes = XKit.any();
        this.excludes = XKit.all();
    }
    
    public XDecryption from(final String xJar) {
        return this.from(new File(xJar));
    }
    
    public XDecryption from(final File xJar) {
        this.xJar = xJar;
        return this;
    }
    
    public XDecryption use(final String password) {
        try {
            this.key = XKit.key(password);
            return this;
        }
        catch (final NoSuchAlgorithmException e) {
            throw new IllegalStateException(e);
        }
    }
    
    public XDecryption use(final String algorithm, final int keysize, final int ivsize, final String password) {
        try {
            this.key = XKit.key(algorithm, keysize, ivsize, password);
            return this;
        }
        catch (final NoSuchAlgorithmException e) {
            throw new IllegalStateException(e);
        }
    }
    
    public XDecryption include(final String ant) {
        this.includes.mix((XEntryFilter)XFilters.ant(ant));
        return this;
    }
    
    public XDecryption include(final Pattern regex) {
        this.includes.mix((XEntryFilter)XFilters.regex(regex.pattern()));
        return this;
    }
    
    public XDecryption exclude(final String ant) {
        this.excludes.mix((XEntryFilter)XFilters.not((XEntryFilter<JarArchiveEntry>)XFilters.ant(ant)));
        return this;
    }
    
    public XDecryption exclude(final Pattern regex) {
        this.excludes.mix((XEntryFilter)XFilters.not((XEntryFilter<JarArchiveEntry>)XFilters.regex(regex.pattern())));
        return this;
    }
    
    public void to(final String jar) throws Exception {
        this.to(new File(jar));
    }
    
    public void to(final File jar) throws Exception {
        if (this.xJar == null) {
            throw new IllegalArgumentException("xJar to decrypt is null. [please call from(String xJar) or from(File xJar) before]");
        }
        if (this.key == null) {
            throw new IllegalArgumentException("key to decrypt is null. [please call use(String password) or use(String algorithm, int keysize, int ivsize, String password) before]");
        }
        XMixEntryFilter<JarArchiveEntry> filter;
        if (this.includes.size() == 0 && this.excludes.size() == 0) {
            filter = null;
        }
        else {
            filter = (XMixEntryFilter<JarArchiveEntry>)XKit.all();
            if (this.includes.size() > 0) {
                filter.mix((XEntryFilter)this.includes);
            }
            if (this.excludes.size() > 0) {
                filter.mix((XEntryFilter)this.excludes);
            }
        }
        XCryptos.decrypt(this.xJar, jar, this.key, (XEntryFilter<JarArchiveEntry>)filter);
    }
}
